@@ -55,7 +55,7 @@ public:
 
 int CustomFormat::parseFormat(const char *psFormat)
 {
-    char achBuf[MAX_LOG_LINE_LEN];
+    char achBuf[4096];
     static const char *s_disallow[] =
     {   "wget", "curl", "bash", "sh", "cat", "more", "less", "strings"    };
 
@@ -78,16 +78,15 @@ int CustomFormat::parseFormat(const char *psFormat)
         const char *key = s_disallow[i];
         const char *f = achBuf;
         const char *p;
-        p = strstr(f, key);
-        while (p)
+        while ((p = strstr(f, key)) != NULL)
         {
             if ((p == achBuf || !isalpha(*(p - 1)))
                 && !isalpha(*(p + strlen(key))))
                 return -1;
-            p += strlen(key);
+            f = p + strlen(key);
         }
     }
-    lstrncpy(achBuf, psFormat, MAX_LOG_LINE_LEN);
+    lstrncpy(achBuf, psFormat, 4096);
 
     char *pEnd = &achBuf[strlen(achBuf)];
     char *p = achBuf;
@@ -97,11 +96,6 @@ int CustomFormat::parseFormat(const char *psFormat)
     int itemId;
     int nonstring_items = 0;
 
-    pEnd = &achBuf[strlen(achBuf)];
-    if (strstr(achBuf, "/bin/") || strstr(achBuf, "/tmp/")
-        || strstr(achBuf, "/etc/"))
-        return -1;
-
     while (1)
     {
         if (state == 0)
@@ -210,9 +204,8 @@ int CustomFormat::parseFormat(const char *psFormat)
                 if (!pBegin)
                     break;
                 *pItemEnd = 0;
-                itemId = HttpHeader::getIndex(pBegin);
-                if ((pItemEnd - pBegin != HttpHeader::getHeaderStringLen(itemId)) ||
-                    (itemId >= HttpHeader::H_HEADER_END))
+                itemId = HttpHeader::getIndex(pBegin, pItemEnd - pBegin);
+                if (itemId >= HttpHeader::H_HEADER_END)
                     itemId = REF_HTTP_HEADER;
                 else
                     pBegin = NULL;
@@ -233,7 +226,7 @@ int CustomFormat::parseFormat(const char *psFormat)
                     break;
                 *pItemEnd = 0;
                 itemId = HttpRespHeaders::getIndex(pBegin);
-                if ((pItemEnd - pBegin != HttpRespHeaders::getHeaderStringLen(
+                if ((pItemEnd - pBegin != HttpRespHeaders::getNameLen(
                                           (HttpRespHeaders::INDEX)itemId))
                     || (itemId >= HttpRespHeaders::H_HEADER_END))
                     itemId = REF_RESP_HEADER;
@@ -340,19 +333,17 @@ int CustomFormat::parseFormat(const char *psFormat)
 }
 
 
-static int logTime(AutoBuf *pBuf, time_t lTime, const char *pFmt)
+static int logTime(char *pBuf, int len, time_t lTime, const char *pFmt)
 {
     struct tm gmt;
     struct tm *pTm = gmtime_r(&lTime, &gmt);
     int n;
-    n = strftime(pBuf->end(), pBuf->available(), pFmt, pTm);
-    pBuf->used(n);
-    return 0;
+    n = strftime(pBuf, len, pFmt, pTm);
+    return n;
 
 }
 
 
-
 static int fixHttpVer(HttpSession *pSession, char *pBuf, int n)
 {
     if (pSession->isHttp2())
@@ -365,87 +356,80 @@ static int fixHttpVer(HttpSession *pSession, char *pBuf, int n)
     return n;
 }
 
-int AccessLog::appendEscape(const char *pStr, int len)
+
+int AccessLog::appendEscape(char *pBuf, int destLen, const char *pStr, int len)
 {
-    const char *pStrEnd = pStr + len;
+    char *pDestEnd = pBuf + destLen;
     char last = '\0';
+    char *p = pBuf;
+    const char *pStrEnd = pStr + len;
     while (pStr < pStrEnd)
     {
         unsigned char ch = *(const uint8_t *)pStr;
 
-        if (m_buf.available() < 10)
-            flush();
         if ((ch < 0x20) || (ch >= 127))
         {
-            m_buf.append('\\');
-            m_buf.append('x');
-            m_buf.append(StringTool::getHex(ch >> 4));
-            m_buf.append(StringTool::getHex(ch));
+            if (p + 5 > pDestEnd)
+                break;
+            *p++ = '\\';
+            *p++ = 'x';
+            *p++ = StringTool::getHex(ch >> 4);
+            *p++ = StringTool::getHex(ch);
         }
         else
         {
+            if (pBuf + 2 > pDestEnd)
+                break;
+
             if ((*pStr == '"') || (*pStr == '\\') || (*pStr == '?' && last == '<'))
-                m_buf.append('\\');
-            m_buf.append(ch);
+                *p++ = '\\';
+            *p++ = ch;
             last = ch;
         }
         ++pStr;
     }
-    return len;
+    return p - pBuf;
 }
 
 
-void AccessLog::appendStrNoQuote(int escape, const char *pStr, int len)
+int AccessLog::appendStrNoQuote(char *pBuf, int len, int escape, const char *pSrc,
+                                int srcLen, AccessLog *pLogger)
 {
-    if (escape)
+    if (pLogger && ((srcLen > 4096)
+                    || (pLogger->m_buf.available() <= srcLen + 100)))
     {
-        appendEscape(pStr, len);
-        return;
-    }
-    if ((len > 4096) || (m_buf.available() <= len + 100))
-    {
-        flush();
-        m_pAppender->append(pStr, len);
+        pLogger->flush();
+        pLogger->m_pAppender->append(pSrc, srcLen);
+        return LS_FAIL;
     }
     else
-        m_buf.append_unsafe(pStr, len);
-}
-
-
-char *AccessLog::appendReqVar(HttpSession *pSession, int id)
-{
-    char *pBuf = m_buf.end();
-    int escape = 0;
-    int n = RequestVars::getReqVar(pSession, id, pBuf, m_buf.available());
-    if (n > 0)
     {
-        if (id == REF_REQ_LINE)
+        if (escape)
+            return appendEscape(pBuf, len, pSrc, srcLen);
+        else if (srcLen > 0)
         {
-            n = fixHttpVer(pSession, pBuf, n);
-            escape = 1;
+            if (srcLen > len)
+                srcLen = len;
+            memmove(pBuf, pSrc, srcLen);
+            return srcLen;
         }
-        if (id < REF_STRING)
-            escape = 1;
-        if (pBuf != m_buf.end())
-            appendStrNoQuote(escape, pBuf, n);
-        else
-            m_buf.used(n);
     }
-    else
-        m_buf.append('-');
-    return pBuf;
+    return 0;
 }
 
 
-//Comments: For internal using, should use pOutBuf == NULL to use internal AutoStr.
-void AccessLog::customLog(HttpSession *pSession, CustomFormat *pLogFmt, bool doFlush)
+int AccessLog::customLog(HttpSession *pSession, CustomFormat *pLogFmt,
+                         char *pOutBuf, int buf_len, AccessLog *pLogger)
 {
     CustomFormat::iterator iter = pLogFmt->begin();
     HttpReq *pReq = pSession->getReq();
     LogFormatItem *pItem;
     const char *pValue = NULL;
-    char *pBuf;
+    char *pBuf = pOutBuf;
+    char *pBufEnd = pBuf + buf_len;
+    char *p;
     int n;
+    int ret;
     int escape;
     while (iter != pLogFmt->end())
     {
@@ -454,24 +438,30 @@ void AccessLog::customLog(HttpSession *pSession, CustomFormat *pLogFmt, bool doF
         switch (pItem->m_itemId)
         {
         case REF_STRING:
-            appendStrNoQuote(escape, pItem->m_sExtra.c_str(), pItem->m_sExtra.len());
+            ret = appendStrNoQuote(pBuf, pBufEnd - pBuf, escape, pItem->m_sExtra.c_str(),
+                                   pItem->m_sExtra.len(), pLogger);
+            if (ret > 0)
+                pBuf += ret;
+            else if (ret < 0)
+                pBuf = pLogger->m_buf.end();
             break;
         case REF_STRFTIME:
             if (pItem->m_sExtra.c_str())
-                logTime(&m_buf, pSession->getReqTime(), pItem->m_sExtra.c_str());
+                pBuf += logTime(pBuf, pBufEnd - pBuf, pSession->getReqTime(),
+                                pItem->m_sExtra.c_str());
             else
             {
-                DateTime::getLogTime(pSession->getReqTime(), m_buf.end());
-                m_buf.used(28);
+                DateTime::getLogTime(pSession->getReqTime(), pBuf);
+                pBuf += 28;
             }
             break;
         case REF_CONN_STATE:
             if (pSession->getStream()->isAborted())
-                m_buf.append('X');
-            else if (pSession->getStream()->isClosing())
-                m_buf.append('-');
+                *pBuf++ = 'X';
+            else if (pSession->getReq()->isKeepAlive())
+                *pBuf++ = '+';
             else
-                m_buf.append('+');
+                *pBuf++ = '-';
             break;
         case REF_COOKIE_VAL:
         case REF_ENV:
@@ -482,10 +472,12 @@ void AccessLog::customLog(HttpSession *pSession, CustomFormat *pLogFmt, bool doF
             case REF_COOKIE_VAL:
                 pValue = RequestVars::getCookieValue(pReq, pItem->m_sExtra.c_str(),
                                                      pItem->m_sExtra.len(), n);
+                escape = 1;
                 break;
             case REF_ENV:
                 pValue = RequestVars::getEnv(pSession, pItem->m_sExtra.c_str(),
                                              pItem->m_sExtra.len(), n);
+                escape = 1;
                 break;
             case REF_HTTP_HEADER:
                 pValue = pReq->getHeader(pItem->m_sExtra.c_str(), pItem->m_sExtra.len(),
@@ -495,32 +487,64 @@ void AccessLog::customLog(HttpSession *pSession, CustomFormat *pLogFmt, bool doF
             case REF_RESP_HEADER:
                 pValue = NULL;
                 n = 0;
-                pSession->getResp()->getRespHeaders().getFirstHeader(
+                pSession->getResp()->getRespHeaders().getHeader(
                     pItem->m_sExtra.c_str(), pItem->m_sExtra.len(), &pValue, n);
                 escape = 1;
                 break;
             }
             if (pValue)
-                appendStrNoQuote(escape, pValue, n);
+            {
+                ret = appendStrNoQuote(pBuf, pBufEnd - pBuf, escape, pValue, n, pLogger);
+                if (ret > 0)
+                    pBuf += ret;
+                else if (ret < 0)
+                    pBuf = pLogger->m_buf.end();
+            }
             else
-                m_buf.append('-');
+                *pBuf++ = '-';
             break;
 
         default:
-            pBuf = appendReqVar(pSession, pItem->m_itemId);
+            p = pBuf;
+            n = RequestVars::getReqVar(pSession, pItem->m_itemId, p, pBufEnd - pBuf);
+            if (n > 0)
+            {
+                if (pItem->m_itemId == REF_REQ_LINE)
+                {
+                    n = fixHttpVer(pSession, p, n);
+                    escape = 1;
+                }
+
+                if (p != pBuf)
+                {
+                    ret = appendStrNoQuote(pBuf, pBufEnd - pBuf, escape, p, n, pLogger);
+                    if (ret > 0)
+                        pBuf += ret;
+                    else if (ret < 0)
+                        pBuf = pLogger->m_buf.end();
+                }
+                else
+                    pBuf += n;
+            }
+            else
+                *pBuf++ = '-';
             break;
 
         }
         ++iter;
     }
-    m_buf.append('\n');
+    *pBuf++ = '\n';
+    return pBuf - pOutBuf;
+}
 
-    if(doFlush)
-    {
-        if ((m_buf.available() < MAX_LOG_LINE_LEN)
-            || !asyncAccessLog())
-            flush();
-    }
+
+void AccessLog::customLog(HttpSession *pSession, CustomFormat *pLogFmt)
+{
+    int n = customLog(pSession, pLogFmt, m_buf.end(), m_buf.available(), this);
+    m_buf.used(n);
+    if ((m_buf.available() < MAX_LOG_LINE_LEN)
+        || !asyncAccessLog())
+        flush();
 }
 
 
@@ -670,6 +694,17 @@ void AccessLog::log(const char *pVHostName, int len, HttpSession *pSession)
 }
 
 
+void AccessLog::appendEscape(const char *data, int len)
+{
+    if (m_buf.available() < 100 + len)
+    {
+        flush();
+        m_buf.guarantee(100 + len);
+    }
+    appendEscape(m_buf.end(), m_buf.available(), data, len);
+}
+
+
 void AccessLog::log(HttpSession *pSession)
 {
     int  n;
@@ -712,6 +747,7 @@ void AccessLog::log(HttpSession *pSession)
     n = fixHttpVer(pSession, pOrgReqLine, n);
     appendEscape(pOrgReqLine, n);
     m_buf.append_unsafe('"');
+    m_buf.append_unsafe(' ');
     m_buf.append_unsafe(
         HttpStatusCode::getInstance().getCodeString(pReq->getStatusCode()), 5);
     if (contentWritten == 0)
@@ -812,23 +848,14 @@ void AccessLog::setRollingSize(off_t size)
 }
 
 
-int  AccessLog::getLogString(HttpSession *pSession, const char *log_pattern, char *pBuf, int bufLen)
+int  AccessLog::getLogString(HttpSession *pSession, const char *log_pattern,
+                             char *pBuf, int bufLen)
+
 {
-    CustomFormat *pLogFmt = parseLogFormat(log_pattern);
+    CustomFormat *pLogFmt = AccessLog::parseLogFormat(log_pattern);
     if (!pLogFmt)
         return -1;
-    customLog(pSession, pLogFmt, false);  //Set flase to not flush the buffer
+    int ret = customLog(pSession, pLogFmt, pBuf, bufLen, NULL);
     delete pLogFmt;
-
-    //Need to copy the accesslog to pBuf
-    int ret = 0;
-    if (m_buf.size() > 0)
-    {
-        if(m_buf.size() < bufLen)
-            bufLen = m_buf.size();
-        memcpy(pBuf, m_buf.begin(), bufLen);
-        ret = bufLen;
-    }
-
     return ret;
 }
