@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -19,7 +19,7 @@
 #include "hiostream.h"
 #include <assert.h>
 #include <lsr/ls_str.h>
-#include <spdy/unpackedheaders.h>
+#include <h2/unpackedheaders.h>
 
 static const ls_str_t s_sProtoName[] =
 {
@@ -44,6 +44,18 @@ const ls_str_t *HioStream::getProtocolName(HiosProtocol proto)
 
 HioStream::~HioStream()
 {
+    if (m_pReqHeaders)
+        delete m_pReqHeaders;
+}
+
+
+void HioStream::reset(int32_t timeStamp)
+{
+    if (m_pReqHeaders)
+        delete m_pReqHeaders;
+    LS_ZERO_FILL(m_pReqHeaders, m_connInfo);
+    StreamStat::reset();
+    setActiveTime(timeStamp);
 }
 
 
