@@ -63,8 +63,6 @@ HttpServerConfig::HttpServerConfig()
     , m_iChildren(1)
     , m_pAdminSock(NULL)
     , m_pGlobalVHost(NULL)
-    , m_bwrap(BWRAP_DISABLED)
-    , m_pBwrapCmdLine(NULL)
 {
     m_pDeniedDir = new DeniedDir();
 }
@@ -120,12 +118,9 @@ void HttpServerConfig::setMaxDynRespHeaderLen(uint32_t len)
 int HttpServerConfig::getSpdyKeepaliveTimeout()
 {
     int timeout = m_iKeepAliveTimeout;
-    if (timeout > 0)
-    {
-        timeout *= ConnLimitCtrl::getInstance().getSslAvailRatio() / 10 + 5;
-        if (timeout > 60)
-            timeout = 60;
-    }
+    timeout *= ConnLimitCtrl::getInstance().getSslAvailRatio() / 10 + 5;
+    if (timeout > 60)
+        timeout = 60;
     return timeout;
 }
 
