@@ -110,7 +110,7 @@ inline int buildStaticFileHeaders(HttpResp *pResp, HttpReq *pReq,
     pResp->getRespHeaders().add(HttpRespHeaders::H_CONTENT_LENGTH, p + 16,
                                 pSendfileInfo->getECache()->getCLHeader().len() - 18);
 
-    pResp->getRespHeaders().appendAcceptRange();
+    pResp->getRespHeaders().addAcceptRanges();
 
     return 0;
 }
@@ -131,7 +131,7 @@ static int processFlvStream(HttpSession *pSession, off_t start)
     {
         HttpResp *pResp = pSession->getResp();
         pSession->resetResp();
-        pSession->addExpiresHeader();
+
         pResp->parseAdd(pData->getFileData()->getHeaderBuf(),
                         pData->getFileData()->getHeaderLen());
 
@@ -352,8 +352,6 @@ int processH264Stream(HttpSession *pSession, double start)
 
     pSession->resetResp();
 
-    pSession->addExpiresHeader();
-
     off_t contentLen = 0;
     if (calcMoovContentLen(pSession, contentLen) == -1)
     {
@@ -518,20 +516,20 @@ int StaticFileHandler::process(HttpSession *pSession,
         }
     }
 
-    char mode = 0;
-    if ((pReq->gzipAcceptable() == GZIP_REQUIRED
-         || pReq->brAcceptable() == BR_REQUIRED)
-        && ((pSession->getSessionHooks()->getFlag(LSI_HKPT_RECV_RESP_BODY)
-             | pSession->getSessionHooks()->getFlag(LSI_HKPT_SEND_RESP_BODY))
-            & LSI_FLAG_DECOMPRESS_REQUIRED) == 0)
-    {
-        mode = (pReq->brAcceptable() == BR_REQUIRED ? SFCD_MODE_BROTLI : 0);
-        if (pReq->gzipAcceptable() == GZIP_REQUIRED)
-            mode |= SFCD_MODE_GZIP;
-    }
-    ret = pInfo->readyCacheData(mode);
-    LS_DBG_L(pReq->getLogSession(), "readyCacheData(%d) return %d",
-             mode, ret);
+    char compressed = (((pReq->gzipAcceptable() == GZIP_REQUIRED)
+                        || (pReq->brAcceptable() == BR_REQUIRED)) &&
+                       ((pSession->getSessionHooks()->getFlag(LSI_HKPT_RECV_RESP_BODY)
+                         | pSession->getSessionHooks()->getFlag(LSI_HKPT_SEND_RESP_BODY))
+                        & LSI_FLAG_DECOMPRESS_REQUIRED) == 0);
+
+    char mode = (pReq->brAcceptable() == BR_REQUIRED ? SFCD_MODE_BROTLI : 0);
+    if (pReq->gzipAcceptable() == GZIP_REQUIRED &&
+        pReq->brAcceptable() != BR_REQUIRED)
+        mode |= SFCD_MODE_GZIP;
+
+    ret = pInfo->readyCacheData(compressed, mode);
+    LS_DBG_L(pReq->getLogSession(), "readyCacheData(%d, %d) return %d",
+             compressed, mode, ret);
     FileCacheDataEx *pECache = pInfo->getECache();
 
     if (!ret)
@@ -552,7 +550,14 @@ int StaticFileHandler::process(HttpSession *pSession,
                 break;
             case SC_200:
                 {
-                    pSession->addExpiresHeader();
+                    const ExpiresCtrl *pExpireDefault = pReq->shouldAddExpires();
+                    if (pExpireDefault)
+                    {
+                        ret = pResp->addExpiresHeader(pCache->getLastMod(),
+                                                      pExpireDefault);
+                        if (ret)
+                            return ret;
+                    }
                     if (pReq->getRedirHdrs())
                     {
                         pResp->parseAdd(pReq->getRedirHdrs(),
@@ -656,7 +661,7 @@ static int buildRangeHeaders(HttpSession *pSession, HttpRange &range)
     StaticFileCacheData *pData = pData1->getFileData();
     off_t bodyLen;
     HttpRespHeaders &buf = pResp->getRespHeaders();
-    pSession->addExpiresHeader();
+
     if (range.count() == 1)
     {
         const char *p = pData->getHeaderBuf();
@@ -736,8 +741,7 @@ static int sendMultipart(HttpSession *pSession, HttpRange &range)
         if (headerLen)
         {
             ret = pSession->writeRespBody(range.getPartHeader(), headerLen);
-            LS_DBG_L(pSession->getLogger(), "[%s] send part header %d bytes",
-                     pSession->getLogId(), ret);
+            LS_DBG_L(pSession, "send part header %d bytes", ret);
 
             if (ret > 0)
             {
