@@ -1,6 +1,6 @@
 /*****************************************************************************
 *    Open LiteSpeed is an open source HTTP server.                           *
-*    Copyright (C) 2013 - 2020  LiteSpeed Technologies, Inc.                 *
+*    Copyright (C) 2013 - 2018  LiteSpeed Technologies, Inc.                 *
 *                                                                            *
 *    This program is free software: you can redistribute it and/or modify    *
 *    it under the terms of the GNU General Public License as published by    *
@@ -25,9 +25,11 @@
 
 CacheEntry::CacheEntry()
     : m_lastAccess(0)
-    , m_iHits(0)
-//     , m_iTestHits(0)
+    , m_lastPurgrCheck(0)
     , m_iMaxStale(0)
+    , m_iHits(0)
+    , m_isDirty(0)
+    , m_isBuilding(0)
     , m_needDelay(0)
     , m_startOffset(0)
     , m_fdStore(-1)
@@ -76,6 +78,12 @@ int CacheKey::getPrivateId(char *pBuf, char *pBufEnd)
     return p - pBuf;
 }
 
+void CacheEntry::setLastPurgrCheck(long tm)
+{
+    m_lastPurgrCheck = tm;
+    g_api->log(NULL, LSI_LOG_DEBUG,
+               "[CACHE] CacheEntry::setLastPurgrCheck called.\n");
+}
 
 int CacheEntry::setKey(const CacheHash &hash, CacheKey *pKey)
 {
